
%    This program estimates the discretized version of a diffusion by the General Method of
%    Moments. The discrete version is not the exact one.
%    It computes initial estimates for the (inexact) discretized diffusion
%    r(t)-r(t-1) = a+br(t-1)+u(t)  with u(t) a zero mean process with conditional variance,
%    (sigma^2)[r(t-1)^2c]


load tipos.prn;
T = size(tipos,1);
lr = tipos(1:T-1,1);
l2r = tipos(1:T-2,1);
dr = tipos(2:T,1)-tipos(1:T-1,1);

y = dr;
xx = [ones(T-1,1) lr];

% Start-up values by MCO on AR(1) model
 [bhat,residuos,sigma2,vcov,estadt,pvalort,R2] = mco_breve(y,xx);
 p = [bhat(1) bhat(2) sigma2 .0]';

%  Compute initial estimates wth Identity weighting matrix
[pcnew,iter,fnew,gnew,hessian] = eemin5('initial',p,'',dr,lr)
p = pcnew;

%  Compute GMM estimates
[pcnew,iter,fnew,gnew,hessian] = eemin5('forqua',p,'',dr,lr)
[qform,A,mh] = forqua(pcnew,dr,lr);

D = zeros(4,4);
D(1,1) = -1;
D(1,2) = -mean(lr);
D(2,1) = -mean(lr);
D(2,2) = -mean(lr.^2);
D(3,3) = -2*(pcnew(3)^2)*mean(lr.^(2*pcnew(4)));
D(3,4) = -2*(pcnew(3)^2)*mean(lr.^(2*pcnew(4)+1));
D(4,3) = D(3,3)*log(pcnew(3));
D(4,4) = -2*(pcnew(3)^2)*mean((lr.^(2*pcnew(4)+1)).*log(lr));

vcov = inv(transpose(D)*A*D)/size(tipos,1)
dt = sqrt(diag(vcov));

display('estimates   std   t-ratios')
[pcnew dt pcnew./dt]
display('minimized objective function')
fnew